# -*- coding: iso-8859-1 -*-
CURRENTLY OUT OF ORDER; This was used to investigate template markup using
a real grammar (which turned out to be too slow).
"""
Code to handle templates.

Templates are a relatively sane part of the wikimedia markup.  Let's
use pyparsing to do them.

This is mainly for grammar development.  In reality, parsing the stuff
is so expensive that we'll need a parser short circuting.
"""

import weakref
import sys
import traceback
import time
import textwrap

from pyparsing import CaselessKeyword, SkipTo, Word, Literal, Suppress,\
	ZeroOrMore, OneOrMore, Forward, Empty, ParserElement, Optional,\
	StringEnd

import wpdata

class ArgSep:
	@staticmethod
	def __str__(*args):
		return "XXX should not be here XXX"

def _joinArguments(toks):
	try:
		args = []
		curParts = []
		for t in toks:
			if t is ArgSep:
				args.append(ParseNode("argument", curParts))
				curParts = []
			else:
				curParts.append(t)
		if curParts:
			args.append(ParseNode("argument", curParts))
		return args
	except:
		traceback.print_exc()
		raise


ParserElement.setDefaultWhitespaceChars(" \t\r")
newline = Literal("\n")
bar = Suppress(Literal("|"))
varOpen = Suppress(Literal("{{{"))
varClose = Suppress(Literal("}}}"))
stOpen = Suppress(Literal("{{#"))
templateOpen = Suppress(Literal("{{"))
templateClose = Suppress(Literal("}}"))
wikilinkOpen = Suppress(Literal("[["))
wikilinkClose = Suppress(Literal("]]"))
stKeyword = ( CaselessKeyword("ifexist") | CaselessKeyword("ifexpr") |
	CaselessKeyword("ifeq") | CaselessKeyword("if") |
	CaselessKeyword("expr") | CaselessKeyword("switch") |
	CaselessKeyword("time") | CaselessKeyword("titleparts") )
startNoInclude = Suppress(CaselessKeyword("<noinclude>"))
endNoInclude = Suppress(CaselessKeyword("</noinclude>"))
startIncludeOnly = Suppress(CaselessKeyword("<includeonly>"))
endIncludeOnly = Suppress(CaselessKeyword("</includeonly>"))
commentStart = Literal("<!--")
commentEnd = Literal("-->")
delims = ( varOpen | varClose | stOpen | templateOpen | templateClose | 
	commentStart | startNoInclude | endNoInclude | startIncludeOnly |
	endIncludeOnly | wikilinkOpen | wikilinkClose | newline )

comment = Suppress(commentStart + SkipTo(commentEnd, include=True))

unparsedText = SkipTo(delims) | StringEnd()
unparsedNoBar = SkipTo( delims | bar )
unparsedNoBar.setName("Unparsed text without bars")

docElement = Forward()
doc = Forward()
argument = Forward()
argList = argument + ZeroOrMore( 
	bar.setParseAction(lambda s, p, t: ArgSep) + argument )
argList.setName("Argument list")

# it seems mediawiki allows wikilinks within arguments and
# ignores bars in there.  What a mess.
wikilink = wikilinkOpen + doc + wikilinkClose
wikilink.setName("Wiki link")

templateVar = (( varOpen + unparsedNoBar + bar + doc + varClose ) |
	( varOpen + unparsedNoBar + varClose ))
templateVar.setName("Variable substitution")
templateFunc = ( stOpen + stKeyword + Suppress(":") +
	argList + templateClose )
templateFunc.setName("Template Function")
template = ( templateOpen + argList + templateClose ) 
template.setName("Template substitution")
noIncludeBlock = startNoInclude + doc + endNoInclude
includeOnlyBlock = startIncludeOnly + doc + endIncludeOnly
docElement << ( newline | templateVar | templateFunc | template | 
	noIncludeBlock | includeOnlyBlock | comment | wikilink )
docElement.setName("docElement")
doc << unparsedText + ZeroOrMore( docElement + unparsedText )
argument << unparsedNoBar + ZeroOrMore( docElement + unparsedNoBar)
argument.setName("Argument")
textWithTemplates = doc + StringEnd()


doc.validate()

class ParseNode(object):
	"""is a node in the parse tree.
	"""
	outputWidth = 80

	def __init__(self, kind, children):
		self.kind, self.children = kind, children
	
	def __str__(self, indent=""):
		res = [indent+"<<%s>>"%self.kind]
		indent = indent+" "
		for child in self.children:
			if isinstance(child, basestring):
				res.append("\n".join(textwrap.wrap(child.strip(), 
					width=self.outputWidth,
					initial_indent=indent, subsequent_indent=indent)))
			else:
				res.append(child.__str__(indent))
		return "\n".join(res)
			

doc.setParseAction(lambda s, pos, toks: ParseNode("doc", toks))
includeOnlyBlock.setParseAction(lambda s, pos, toks: ParseNode("incOnly", toks))
noIncludeBlock.setParseAction(lambda s, pos, toks: ParseNode("noInc", toks))
template.setParseAction(lambda s, pos, toks: ParseNode("template", toks))
templateFunc.setParseAction(lambda s, pos, toks: ParseNode("tFunc", toks))
templateVar.setParseAction(lambda s, pos, toks: ParseNode("tVar", toks))
argList.setParseAction(lambda s, pos, toks: _joinArguments(toks))


def parse(txt):
	return textWithTemplates.parseString(txt)

def setDebug(debug):
	templateVar.setDebug(debug)
	argList.setDebug(debug)
	unparsedNoBar.setDebug(debug)
	argument.setDebug(debug)
	templateFunc.setDebug(debug)
	wikilink.setDebug(debug)
	template.setDebug(debug)

_specials = {
	"!":  "|",
	"!-": "|-",
	"!!": "||",
	"!-!":"|-|",
	"!+": "|+",
	"!~": "|-!",
	"(!": "{|",
	"!)": "|}",
}

def getSpecialTemplate(token):
	"""returns the substitution for the predefined template names.
	"""
	return _specials.get(token)


class TemplateFunc:
	"""is a template function (if, ifexpr, etc).
	"""
	def __init__(self, parent, args):
		self.parent = weakref.proxy(parent)
		self.kw = args[0]
		self.args = _joinArguments(args[1:])

	def __repr__(self):
		return "TemplateFunc %s"%self.kw

	def _expand_if(self):
		if self.args[0].strip():
			return self.args[1]
		else:
			return self.args[2]
	
	def _expand_ifeq(self):
		if self.args[0].strip()==self.args[1].strip():
			return self.args[2]
		else:
			return self.args[3]
	
	def _expand_ifexist(self):
		if wpdata.getWpData().has_key(self.args[0].strip()):
			return self.args[1]
		else:
			return self.args[2]
	
	def _expand_expr(self):
		# XXX TODO: implement this, either by (security issue!) handing that
		# args[0] over to eval after doing round, div, etc, or by really
		# parsing the expression.
		return "0"
	
	def _expand_ifexpr(self):
		if float(self._expand_expr()):
			return self.args[1]
		else:
			return self.args[2]
	
	def _expand_switch(self):
		def parseCases(args):
			default = ""
			caseDict = {}
			curHeads = []
			for arg in args:
				if "=" in arg:
					newHead, tail = arg.split("=", 1)
					curHeads.append(newHead.strip())
					for head in curHeads:
						if head=="#default":
							default = tail
						else:
							caseDict[head] = tail
					curHeads = []
				else:
					curHeads.append(arg.strip())
			return caseDict, default
		caseDict, default = parseCases(self.args[1:])
		print ">>>>", caseDict
		return caseDict.get(self.args[0].strip(), default)
	
	def _expand_time(self):
		# XXX TODO: implement this
		if len(self.args)==2:
			return self.args[1]
		else:
			return time.asctime()

	def _expand_titleparts(self):
		numParts = int(self.args[1])
		if len(self.args)>2:
			startIndex = int(self.args[2])-1
		else:
			endIndex = 0
		if numParts==0:
			return self.args[0]
		else:
			return self.args[0].strip().split("/")[startIndex:startIndex+numParts]

	def expand(self):
		try:
			return getattr(self, "_expand_"+self.kw)()
		except:
			traceback.print_exc()
			raise


class TemplateHandler:
	"""is a class that encapsulates the information needed while
	parsing and formatting a template.
	"""
	def __init__(self, name, args, isToplevel=False):
		self.name = name
		self.expandedValue = getSpecialTemplate(self.name)
		if self.expandedValue==None:
			self._parseArguments(args)
			self._setParseActions()

	def _parseArgument(self, arg):
		if "=" in arg:
			return [s.strip() for s in arg.split("=", 1)]
		else:
			return None, arg.strip()

	def _parseArguments(self, argSeq):
		self.positionalArgs = []
		self.kwArgs = {}
		for rawArg in argSeq:
			argName, argVal  = self._parseArgument(rawArg)
			if argName:
				self.kwArgs[argName] = argVal
			self.positionalArgs.append(argVal)
		print ">>>> parsed arguments:", self.positionalArgs, self.kwArgs

	def _getRawContent(self):
		title = self.name
		if not ":" in title:
			title = "%s:%s"%(wpprefs.getLangData("templateNamespace"), title)
		return wpdata.getWpData().getArticle(title)

	def _makeParseAction(self, callable):
		def parseAction(s, pos, toks):
			try:
				return callable(toks)
			except Exception, msg:
				sys.stderr.write(">>>>>>>>>>>>> Panic\n")
				traceback.print_exc()
				raise
		return parseAction

	def _setParseActions(self):
		templateVar.setParseAction(self._makeParseAction(self._parseTemplateVar))
		templateFunc.setParseAction(self._makeParseAction(self._parseTemplateFunc))
		noIncludeBlock.setParseAction(lambda s, pos, toks: "")

	def _parseTemplateVar(self, toks):
		toks = _joinArguments(toks)
		if len(toks)==1:
			varName, default = toks[0], ""
		else:
			varName, default = toks
		if self.kwArgs.has_key(varName):
			return self.kwArgs[varName]
		try:
			return self.positionalArgs[int(varName)]
		except (IndexError, ValueError):
			return default

	def _parseTemplateFunc(self, toks):
		return TemplateFunc(self, toks)

	def _realExpand(self):
#		try:
#			content = self._getRawContent()
#		except PageNotFound:
#			return ('<span style="errmsg">Include %s could not'
#				' be satisfied</span>'%(repr(title)))
		parts = []
		bla = textWithTemplates.parseString(open("zw.txt").read().decode("utf-8"))
		print ">>>>>>>>>>>>>>>><<", bla
		for child in bla:
			if isinstance(child, basestring):
				parts.append(child)
			else:
				try:
					parts.append(child.expand())
				except:
					traceback.print_exc()
					raise
		return " ".join(parts)

	def expand(self):
		if self.expandedValue==None:
			self.expandedValue = self._realExpand()
		return self.expandedValue


def _test():
	import sys
	textWithTemplates.parseString("")
	textWithTemplates.parseString("{{{Foo}}}")
	textWithTemplates.parseString("{{{Foo|}}}")
	textWithTemplates.parseString("{{#if: x | bla}}")
	textWithTemplates.parseString("{{#switch: bar | bla | "
		" foo=bar | bar=x}}")
	textWithTemplates.parseString("{{#switch: bar | bla | "
		" foo=bar | bar=x | #default=baz}}")
	textWithTemplates.parseString("{{#switch: bar | bla\n | "
		" foo=bar\n | bar=x\n | #default=baz}}")
	textWithTemplates.parseString("{{#switch: {{lc:{{{Wappen}}}}} | "
		"bla\n | "
		" foo=bar\n | bar=x\n | #default=baz}}")
	textWithTemplates.parseString("{{#switch: {{lc:{{{Wappen}}}}} | "
		"[[Bild:Führt kein Wappen.svg|120px|Quatsch]]\n | "
		" foo=bar\n | bar=x\n | #default=baz}}")
	textWithTemplates.parseString("{{#if: x | bla | [[foo|x|y]]<br />}}")
	textWithTemplates.parseString("{{#if: {{{Wappen|}}} | foo | bar}}\n")
	setDebug(True)
	sys.exit()

if __name__=="__main__":
	#_test()
	#setDebug(True)
	print doc.parseString(open("zw.txt").read())[0]
