import Tkinter
import Queue
import threading
import operator
import select
import wpprefs
import wpserver
import BaseHTTPServer

from smallwp import wpdata

class Logshower(Tkinter.Frame):
	"""is a widget containing a scrolling display of up to maxLines lines.
	"""
	def __init__(self, master, maxLines=70):
		Tkinter.Frame.__init__(self, master)
		self.maxLines = maxLines
		self.display = Tkinter.Text(self,
			borderwidth=0, wrap=Tkinter.WORD, relief=Tkinter.FLAT)
		self.display.bind("<Key>", lambda ev: "break")
		self.scroller = Tkinter.Scrollbar(self, command=self.display.yview)
		self.display.config(yscrollcommand=self.scroller.set)
		self.display.grid(row=0, column=0, sticky=Tkinter.NW+Tkinter.SW)
		self.scroller.grid(row=0, column=1, sticky=Tkinter.NW+Tkinter.SW)
		self.columnconfigure(0, weight=1)
		self.rowconfigure(0, weight=1)
	
	def addLine(self, line):
		self.display.insert(Tkinter.END, "\n"+line)
		self.display.see(Tkinter.END)
		numberOfLines = int(self.display.index(Tkinter.END).split(".")[0])
		if numberOfLines>self.maxLines:
			self.display.delete("0.0", 
				"%d.0"%(numberOfLines-self.maxLines))


class Statusbar(Tkinter.Label):
	def __init__(self, master):
		Tkinter.Label.__init__(self, master, justify=Tkinter.LEFT, 
			relief=Tkinter.SUNKEN, anchor=Tkinter.W)
		self.afterId = None

	def _clearMsg(self):
		self.afterId = None
		self.config(text="")

	def showMsg(self, msg, timeout=None):
		if self.afterId:
			self.after_cancel(self.afterId)
		self.config(text=msg)
		if timeout:
			self.afterId = self.after(timeout*1000, self._clearMsg)


class WpGui(Tkinter.Tk):
	"""is a simple GUI for the wikipedia server.
	"""
	LOG = 1
	BAR = 2
	def __init__(self):
		Tkinter.Tk.__init__(self)
		self.title(wpprefs.getPref("sitename"))
		self._buildWidgets()
		self.bind("q", lambda ev: self.quit())
		self.focus()
	
	def _buildWidgets(self):
		self._buildMenubar()
		self.logshower = Logshower(self)
		self.logshower.grid(row=1, column=1, sticky=Tkinter.SW+Tkinter.NE)
		self.statusbar = Statusbar(self)
		self.statusbar.grid(row=2, column=1, sticky=Tkinter.NW+Tkinter.SE)
		self.rowconfigure(1, weight=1)
		self.columnconfigure(1, weight=1)

	def _buildMenubar(self):
		menubar = Tkinter.Menu(self)
		filemenu  = Tkinter.Menu(menubar, tearoff=0)
		filemenu.add_command(label="Quit", command=self.quit)
		menubar.add_cascade(label="File", menu=filemenu)
		self.config(menu=menubar)

	def _logMessage(self, message):
		self.logshower.addLine(message)

	def _changeStatusbar(self, message, timeout=None):
		self.statusbar.showMsg(message, timeout)

	def send(self, type, *payload):
		self.methods[type](self, *payload)

	methods = {
		LOG: _logMessage,
		BAR: _changeStatusbar,
	}


class WpServerAdapter:
	def __init__(self, ui):
		self.ui = ui
		self.stop = False
		server_address = (wpprefs.getPref("address"), 
			wpprefs.getPref("port"))
		self.httpd = BaseHTTPServer.HTTPServer(server_address, 
			wpserver.WpHandler)
		self.httpd.data = wpdata.makeData()
		self.httpd.log_message = self.logToUi
		self.serverThread = threading.Thread(target=self.runServer)
		self.serverThread.setDaemon(True)
		self.serverThread.start()
		self.ui.send(self.ui.BAR, "Listening on %s:%d"%server_address, 20)

	def logToUi(self, msg, *args):
		self.ui.send(self.ui.LOG, msg%args)

	def runServer(self):
		while not self.stop:
			rdyFds = reduce(operator.add,
				select.select([self.httpd.fileno()], [self.httpd.fileno()],
					[self.httpd.fileno()], 0.1))
			if rdyFds:
				self.httpd.handle_request()
	
	def stopServer(self):
		self.ui.send(self.ui.BAR, "Shutting down", 20)
		self.stop = True
		self.serverThread.join(10)
		

def main():
	gui = WpGui()
	server = WpServerAdapter(gui)
	gui.geometry("600x150")
	gui.mainloop()
	server.stopServer()


if __name__=="__main__":
	main()
