#!/bin/sh
# some very basic testing.  This needs XML and HTML dumps of the
# ast wikipedia in the current directory.
# The software needs to be installed (probably via setup.py develop)

XMLDUMP=astwiki-20090122-pages-articles.xml.bz2
HTMLDUMP=wikipedia-ast-html.tar.7z
export TESTDIR=`pwd`
export SMALLWPCONF=$TESTDIR/config-test

cat > config-test <<EOF
[general]
baseDir=$TESTDIR/data
doTemplates=True
EOF

die() {
	echo $@
	exit 1
}

runTest() {
	# Args: URL, string to find in response, error message
	wget -q -O - "$1" \
		| grep "$2" > /dev/null \
		|| die "FAIL: $3, $1"
}


commandTest() {
	### some quick UI tests
	splitWp -H | grep "wikipedia data" > /dev/null \
		|| die "Config help didn't show up"
}

htmlTest() {
	### test with HTML dump
	splitWp $HTMLDUMP || die "Split HTML failed"
	wpTui > /dev/null 2>&1&
	serverPID=$!
	sleep 5  # Let the server come up.  Yes, this is cheesy.
	runTest http://localhost:8780/ curFold \
		"Index page does not seem to contain sections."
	runTest 'http://localhost:8780/opensection/5#curFold' 'id="titleFrag"'\
		"Level 1 section does not seem to exist."
	runTest 'http://localhost:8780/pages/212 Medea' 'href="/q/Asteroide.html"' \
		"Some random page looks ok."
	runTest localhost:8780/css/smallwp.css input.scalable "Trouble with CSS."
	runTest http://localhost:8780/q/Asteroide.html '"/q/Planeta_nanu.html"' \
		"Wiki link delivered"
	kill $serverPID
	rm -rf $TESTDIR/data
}

xmlTest() {
	### test with XML dump
	splitWp $XMLDUMP || die "Split XML failed"
	wpGui 2>&1 > /dev/null&
	serverPID=$!
	sleep 7  # Let the server come up.  Yes, this is cheesy.
	runTest http://localhost:8780/ curFold \
		"Index page does not seem to contain sections."
	runTest 'http://localhost:8780/opensection/5#curFold' 'id="titleFrag"'\
		"Level 1 section does not seem to exist."
	runTest 'http://localhost:8780/pages/212 Medea' \
		'<a href="/pages/asteroide">asteroide</a>' \
		"Some random page looks ok."
	runTest http://localhost:8780/pages/Asturies \
		'<strong>Comunid.. Aut..noma del </strong> <br />'\
		"Templates appear to be somewhat expanded."
	runTest "http://localhost:8780/search/?q=quel" 'Pilsner+Urquell">Pilsner '\
		"RE search appears to work"
	kill $serverPID
}

commandTest
htmlTest
xmlTest
echo "All tests run."
